Функция доступна в провайдере версии 4.5.1 и выше.

_Data source_ функция **decort_cb_rg** служит для получения информации об уже существующей ресурсной группе.

Ресурсная группа в облачной платформе DECORT это способ группирования _compute_-ов (например, по функциональному признаку или принадлежности к одному и тому же проекту). Ресурсную группу можно рассматривать как небольшой персональный дата-центр, в котором размещаются один или несколько серверов и виртуальных сетевых сегментов (ViNS).

Экземпляры _compute_ (виртуальные серверы) всегда создаются в принадлежности к какой-либо ресурсной группе. Как следствие, для создания экземпляра _compute_ помимо других параметров требуется указать идентификатор целевой ресурсной группы. Для получения такого идентификатора можно воспользоваться рассматриваемой здесь _data source_ функцией **decort_cb_rg**.

## Аргументы
_Data source_ функция **decort_cb_rg** принимает следующие аргументы:

| Аргумент | Тип | Обязательный | Описание                                                                                                         |
| --- | --- | --- |------------------------------------------------------------------------------------------------------------------|
| rg_id | int | :heavy_check_mark: | Идентификатор ресурсной группы, по которой требуется получить информацию. Идентификатор не может быть нулевым.   |
| reason | string | :x: | Причина запроса                                                                                                  |

## Возвращаемые значения
В случае успешного выполнения _data source_ функция **decort_cb_rg** возвращает в указанную при вызове переменную следующие значения:

| Параметр                 | Тип                                                      | Описание                                                                                                                                                                                                                                                                                                                                                                                                                                                                |
|--------------------------|----------------------------------------------------------|-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| account_id               | int                                                      | Уникальный идентификатор подписчика-владельца данной ресурсной группы.                                                                                                                                                                                                                                                                                                                                                                                                  |
| account_name             | string                                                   | Имя подписчика ("account"), которому принадлежит данная ресурсная группа.                                                                                                                                                                                                                                                                                                                                                                                               |
| acl                      | []Struct [см. ниже](#описание-структуры-acl)             | Cписок Acces Control                                                                                                                                                                                                                                                                                                                                                                                                                                                    |
| cpu_allocation_parameter | string                                                   | Параметр выделения CPU                                                                                                                                                                                                                                                                                                                                                                                                                                                  |
| cpu_allocation_ratio     | float                                                    | Отношение выделения CPU                                                                                                                                                                                                                                                                                                                                                                                                                                                 |
| created_by               | string                                                   | Пользователь - создатель ресурсной группы                                                                                                                                                                                                                                                                                                                                                                                                                               |
| created_time             | int                                                      | Время создания ресурсной группы                                                                                                                                                                                                                                                                                                                                                                                                                                         |
| def_net_id               | int                                                      | Уникальный идентификатор сети, установленной по умолчанию для данной ресурсной группы. Экземпляры _compute_, создаваемые в данной ресурсной группе, по умолчанию будут иметь подключение к данной сети, если при их создании явно не указано иное.<br/>Для `def_net_type = "EXTNET" в этом параметре указан идентификатор внешней сети. Для `def_net_type = "PRIVATE" - идентификатор ViNS.                                                                             |
| def_net_type             | string                                                   | Тип сети, установленной по умолчанию для данной ресурсной группы. Может принимать одно из следующих значений:<br> * EXTNET - подключение во внешнюю сеть с идентификатором `def_net_id`<br> * PRIVATE - подключение в виртуальный сетевой сегмент (ViNS ) с идентификатором `def_net_id`<br> * NONE - сеть по умолчанию не задана, экземпляры _compute_ в данной ресурсной группе будут создаваться без сетевых подключений, если при их создании явно не указано иное. |
| deleted_by               | string                                                   | Информация о том, кто удалил группу                                                                                                                                                                                                                                                                                                                                                                                                                                     |
| deleted_time             | int                                                      | Время удаления группы                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| desc                     | string                                                   | Текстовое описание данной ресурсной группы.                                                                                                                                                                                                                                                                                                                                                                                                                             |
| dirty                    | bool                                                     | Dirty                                                                                                                                                                                                                                                                                                                                                                                                                                                                   |
| gid                      | int                                                      | GRID ID                                                                                                                                                                                                                                                                                                                                                                                                                                                                 |
| guid                     | int                                                      | ID ресурса                                                                                                                                                                                                                                                                                                                                                                                                                                                              |
| lock_status              | string                                                   | Статус доступности                                                                                                                                                                                                                                                                                                                                                                                                                                                      |
| milestones               | int                                                      | Вехи                                                                                                                                                                                                                                                                                                                                                                                                                                                                    |
| name                     | string                                                   | Имя данной ресурсной группы. Обратите внимание, что имя ресурсной группы является уникальным только в рамках одного и того же подписчика ("account").                                                                                                                                                                                                                                                                                                                   |
| rg_id                    | int                                                      | Идентификатор данной ресурсной группы.                                                                                                                                                                                                                                                                                                                                                                                                                                  |
| register_computes        | bool                                                     | Флаг регистрации ВМ во внешней системе                                                                                                                                                                                                                                                                                                                                                                                                                                  |
| resource_limits          | []Struct [см. ниже](#описание-структуры-resource_limits) | Лимиты ресурсов                                                                                                                                                                                                                                                                                                                                                                                                                                                         |
| resource_types           | []string                                                 | Типы ресурсов                                                                                                                                                                                                                                                                                                                                                                                                                                                           |
| secret                   | string                                                   | Секрет                                                                                                                                                                                                                                                                                                                                                                                                                                                                  |
| status                   | string                                                   | Статус ресурсной группы                                                                                                                                                                                                                                                                                                                                                                                                                                                 |
| uniq_pools               | []string                                                 | Список пулов. Например, ["sep1_poolName1", "sep2_poolName2"]                                                                                                                                                                                                                                                                                                                                                                                                            |
| updated_by               | string                                                   | Кем был обновлен ресурс                                                                                                                                                                                                                                                                                                                                                                                                                                                 |
| updated_time             | int                                                      | Время последнего обновления ресурса                                                                                                                                                                                                                                                                                                                                                                                                                                     |
| vins                     | []int                                                    | Список виртуальных сетей                                                                                                                                                                                                                                                                                                                                                                                                                                                |
| computes                 | []int                                                    | Список id компьютов                                                                                                                                                                                                                                                                                                                                                                                                                                                     |

### Описание структуры acl
| Параметр | Тип | Описание |
| --- | --- | --- |
| explicit | bool | Видимость аккаунта |
| guid | string | GUID |
| right | string | Права |
| status | string | Статус |
| type | string | Тип |
| user_group_id | string | ID группы юзеров |

### Описание структуры resource_limits
| Параметр | Тип | Описание |
| --- | --- | --- |
| cu_c | float | Количество ядер CPU |
| cu_d | float | Размер диска |
| cu_dm | float | Использованный размер хранилища виртуального диска |
| cu_i | float | Количество публичных IP адресов |
| cu_m | float | размер RAM |
| cu_np | float | Хранилище траффика |
| gpu_units | float | количество GPU ядер |

## Пример использования
Пример вызова _data source_ функции **decort_cb_rg**:

```terraform
data "decort_cb_rg" "target_rg" {
  rg_id = 1022
  #reason = "TEST"
}
# Once the above directive completes, resource group ID will be accessible as
# data.decort_cb_rg.target_rg.rg_id
```

Данный пример подразумевает, что ранее по тексту tf-файла выполнена корректная инициализация провайдера (подробнее об инициализации Terraform провайдера DECORT см. соответствующий [раздел](https://repository.basistech.ru/BASIS/terraform-provider-decort/src/branch/main/wiki/4.5.2/04.02-Инициализация-Terraform-провайдера-DECORT.md)). 

После успешного завершения такого вызова **decort_cb_rg** в переменной `data.decort_cb_rg.target_rg` будут сохранены возвращаемые значения. В частности, для получения идентификатора ресурсной группы следует использовать конструкцию `data.decort_cb_rg.target_rg.rg_id`
