Функция доступна в провайдере версии 4.5.1 и выше.

_Data source_ функция **decort_cb_rg_list_computes** служит для получения информации о компьютах ресурсной группы.

Ресурсная группа в облачной платформе DECORT это способ группирования _compute_-ов (например, по функциональному признаку или принадлежности к одному и тому же проекту). Ресурсную группу можно рассматривать как небольшой персональный дата-центр, в котором размещаются один или несколько серверов и виртуальных сетевых сегментов (ViNS).

Экземпляры _compute_ (виртуальные серверы) всегда создаются в принадлежности к какой-либо ресурсной группе. Как следствие, для создания экземпляра _compute_ помимо других параметров требуется указать идентификатор целевой ресурсной группы. Для получения такого идентификатора можно воспользоваться рассматриваемой здесь _data source_ функцией **decort_cb_rg_list_computes**.

## Аргументы
_Data source_ функция **decort_cb_rg_list_computes** принимает следующие аргументы:

| Аргумент | Тип | Обязательный | Описание |
| --- | --- | --- | --- |
| rg_id | int | :heavy_check_mark: | Идентификатор ресурсной группы |
| compute_id | int | :x: | Фильтр по ID компьюта |
| name | string | :x: | Фильтр по имени компьюта |
| account_id | int | :x: | Фильтр по ID аккаунта |
| tech_status | string | :x: | Фильтр по тех. статусу |
| status | string | :x: | Фильтр по статусу |
| ip_address | string | :x: | Фильтр по IP адресу |
| extnet_name | string | :x: | Фильтр по имени extnet |
| extnet_id | int | :x: | Фильтр по ID extnet |
| page | int | :x: | Номер страницы |
| size | int | :x: | Размер страницы |

Пользователь облачной платформы, от лица которого происходит обращение к данной функции, должен иметь доступ к указанному `account` и ресурсной группе. В противном случае возникнет ошибка доступа.

Пользователь облачной платформы задается на стадии инициализации Terraform провайдера DECORT (см. [подробности](https://repository.basistech.ru/BASIS/terraform-provider-decort/src/branch/main/wiki/4.5.3/04.02-Инициализация-Terraform-провайдера-DECORT.md)).

## Возвращаемые значения
В случае успешного выполнения _data source_ функция **decort_cb_rg_list_computes** возвращает в указанную при вызове переменную, в поле _items_, массив, каждый элемент которого состоит из следующих параметров:

| Параметр            | Тип                                                     | Описание                                                                             |
|---------------------|---------------------------------------------------------|--------------------------------------------------------------------------------------|
| account_id          | int                                                     | Уникальный идентификатор подписчика-владельца ("account") данной виртуальной машины. |
| account_name        | string                                                  | Имя подписчика, которому принадлежит данная виртуальная машина.                      |
| affinity_label      | string                                                  | Лейбл аффинити                                                                       |
| affinity_rules      | []Struct [см. ниже](#описание-структуры-affinity-rules) | Правила аффинити для компьютов                                                       |
| affinity_weight     | int                                                     | weight аффинити                                                                      |
| anti_affinity_rules | []Struct [см. ниже](#описание-структуры-affinity-rules) | Правила анти аффинити для компьютов                                                  |
| cpus                | int                                                     | Количество виртуальных CPU, выделенных данному серверу.                              |
| created_by          | string                                                  | Кем создан ресурс                                                                    |
| created_time        | int                                                     | Время создания ресурса                                                               |
| deleted_by          | string                                                  | Кем удален ресурс                                                                    |
| deleted_time        | int                                                     | Время удаления ресурса                                                               |
| id                  | int                                                     | Идентификатор компьюта                                                               |
| name                | string                                                  | Имя виртуального сервера                                                             |
| ram                 | int                                                     | Объём оперативной памяти в МБ, выделенной этой виртуальной машине.                   |
| registered          | bool                                                    | Компьют зарегестрирован                                                              |
| rg_name             | string                                                  | Имя ресурсной группы, к которой принадлежит виртуальная машина.                      |
| rg_id               | int                                                     | ID ресурсной группы                                                                  |
| status              | string                                                  | Статус диска                                                                         |
| tech_status         | string                                                  | Технический статус диска                                                             |
| total_disk_size     | int                                                     | Размер диска                                                                         |
| updated_by          | string                                                  | Кем обновлен ресурс                                                                  |
| updated_time        | int                                                     | Время обновления ресурса                                                             |
| user_managed        | bool                                                    | User managed                                                                         |
| vins_connected      | int                                                     | Кол-во присоединенных винсов                                                         |

### Описание структуры affinity rules
| Параметр | Тип | Описание |
| --- | --- | --- |
| guid | string | guid |
| key | string | Ключ |
| mode | string | Режим сравнения |
| policy | string | Степень "строгости" этого правила |
| topology | string | Топология |
| value | string | Значение |

## Пример использования
Пример вызова _data source_ функции **decort_cb_rg_list_computes**:

```terraform
data "decort_cb_rg_list_computes" "list_computes" {
  rg_id = 123
  #compute_id = 100
  #name = "test"
  #account_id = 100
  #tech_status = "STARTED"
  #status = "ENABLED"
  #ip_address = "1.1.1.1.1"
  #extnet_name = "test"
  #extnet_id = 100
  #page = 1
  #size = 1
}
# Once the above directive completes, resource group computes will be accessible as
# data.decort_cb_rg_list_computes.list_computes.items
```

Данный пример подразумевает, что ранее по тексту tf-файла выполнена корректная инициализация провайдера (подробнее об инициализации Terraform провайдера DECORT см. соответствующий [раздел](https://repository.basistech.ru/BASIS/terraform-provider-decort/src/branch/main/wiki/4.5.3/04.02-Инициализация-Terraform-провайдера-DECORT.md) и [пример](https://repository.basistech.ru/BASIS/terraform-provider-decort/src/branch/main/wiki/4.5.3/02.-Пример-работы.md)). 

После успешного завершения такого вызова **decort_cb_rg_list_computes** в переменной `data.decort_cb_rg_list_computes.list_computes.items` будут сохранены возвращаемые значения.
