Функция доступна в провайдере версии 2.0 и выше.

_Resource_ функция **decort_virtual_image** служит для управления виртуальным образом. Виртуальный образа создаются на основе уже созданных ранее образов, таким образом, они привязываются к уже созданным образам. Образ, на основе которого создан виртуальный диск, удалить невозможно - для этого необходимо удалить виртуальный образ, либо связать его с другим образом.

## Аргументы
_Resource_ функция **decort_virtual_image** для создания виртуального образа принимает следующие аргументы:

| Аргумент | Тип | Обязательный | Описание |
| --- | --- | --- | --- |
| name | string | :heavy_check_mark: | Имя виртуального образа |  
| link_to | int | :heavy_check_mark: | ID образа, на основе котрого будет создан виртуальный |
| permanently | bool | :x: | Мгновенное удаление<br/>можно использовать перед удалением |

## Возвращаемые значения
В случае успешного выполнения _resource_ функция **decort_virtual_image** возвращает в указанную при вызове переменную следующие значения:

| Параметр | Тип | Описание |
| --- | --- | --- |
| unc_path | string | unc path |
| \_ckey | string | ckey |
| account_id | int | accountId привязки образа |
| architecture | string | Архитектура образа, возможен вариант X86_64 или PPC64_LE |
| boot_type | string | Тип запуска ОС - bios или uefi |
| bootable | bool | Является ли образ загрузочным |
| computeci_id | int | ID вычислительного ресурса |
| deletedTime | int | Время удаления | 
| desc | string | Описание образа |
| drivers | []string | Cписок используемых драйверов |
| enabled | bool | Доступность образа |
| gid | int | ID платформы, где образ будет создан |
| guid | int | Внутренний id образа, совпадает с image_id |
| history | []Struct [см. ниже](#описание-структуры-history) | История обновлений образа |
| hot_resize | bool | Использует ли машина горячее изменение размера образа |
| image_id | int | ID образа |
| image_type | string | Тип образа: linux, windows или другой |
| last_modified | int | Дата последнего изменения |
| link_to | int | Привязка к образу, используется в виртуальных образах |
| milestones | int | Вехи |
| name | string | Название образа |
| password | string | Пароль для образа |
| permanently | bool | Удаление образа: мгновенное или нет |
| pool_name | string | Пул, в котором был создан образ |
| present_to | []int | Список ID ресурсов, которым предоставлен образ |
| provider_name | string | Наименование провайдера |
| purge_attempts | int | Попытки |
| reference_id | string | ID ссылки |
| res_id | string | ID ресурса |
| rescuecd | bool | CD или нет |
| sep_id | int | storage endpoint provider ID |
| shared_with | []int | Список ресурсов, имеющих доступ к образу |
| size | int | Размер образа |
| status | string | Статус образа |
| tech_status | string | Технический статус |
| username | string | Имя пользователя образа |
| version | string | Версия |

### Описание структуры history 
| Параметр | Тип | Описание |
| --- | --- | --- |
| id | int | Идентификатор образа |
| guid | string | Внутренний id образа |
| timestamp | int | Время обновления образа |

## Пример использования
Пример вызова _resource_ функции **decort_virtual_image**:
```terraform
resource "decort_image_virtual" "iv" {
  #имя виртуального образа
  #обязательный параметр
  #при изменении - обновляет имя виртуального образа
  #тип - строка
  name = "iv-name"

  #id реального образа, на который будет ссылаться виртуальный
  #обязательный параметр
  #при изменении - ссылается на другой реальный образ
  #тип - целое число
  link_to = 123

  #флаг для удаления диска
  #опциональный параметр
  #тип - булев
  permanently = false
}
```

Данный пример подразумевает, что ранее по тексту tf-файла выполнена корректная инициализация провайдера (подробнее об инициализации Terraform провайдера DECORT см. соответствующий [раздел](https://repository.basistech.ru/BASIS/terraform-provider-decort/src/branch/main/wiki/4.6.0/04.02-Инициализация-Terraform-провайдера-DECORT.md)). 

После успешного завершения такого вызова **decort_virtual_image** в переменной `decort_virtual_image.iv` будут сохранены возвращаемые значения.
